package com.shanghaichengdi.ghjgitem.controller;


import cn.hutool.json.JSON;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageHelper;
import com.google.common.hash.HashCode;
import com.shanghaichengdi.ghjgitem.constant.HttpStatus;
import com.shanghaichengdi.ghjgitem.domain.*;
import com.shanghaichengdi.ghjgitem.mapper.TransportSgBaseNumberMapper;
import com.shanghaichengdi.ghjgitem.mapper.TransportSgDwNzxAgentMapper;
import com.shanghaichengdi.ghjgitem.service.*;
import com.shanghaichengdi.ghjgitem.util.CommonTools;
import com.shanghaichengdi.ghjgitem.util.StringUtils;
import com.shanghaichengdi.ghjgitem.vo.ObjectBodyVo;
import com.shanghaichengdi.ghjgitem.vo.SyncVo;
import com.shanghaichengdi.ghjgitem.vo.response.ResultRes;
import com.shanghaichengdi.ghjgitem.vo.response.ReturnMsg;
import com.shanghaigeography.Util.DateUtils;
import com.shanghaigeography.Util.ObjOPUtil;
import com.shanghaigeography.Util.SnowflakeConfig;
import lombok.extern.slf4j.Slf4j;
import org.apache.shiro.crypto.hash.Hash;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import java.io.IOException;
import java.math.BigDecimal;
import java.util.*;
import java.util.function.Function;
import java.util.stream.Collectors;

@RestController
@CrossOrigin
@Slf4j
@RequestMapping("/sipg")
public class SIPGDataTransportController extends CommonController  {


    @Autowired
    private TransportSgDwNzxBoatService boatService;

    @Autowired
    SnowflakeConfig snowflakeConfig;


    @Autowired
    private TransportSgBaseNumberService baseNumberService;
    /**
     *  船舶载运航次数据
     * @param vo
     * @return
     */
    @PostMapping("/putShipVoyageData")
    public ReturnMsg getConnRecordInfoListSync(@RequestBody @Validated ObjectBodyVo vo) {
        if (!checkDbKey(vo.getKey())) {
            return ReturnMsg.error(HttpStatus.BAD_REQUEST, "key值有误,请确认您输入的key值正确!");
        }
        String paramJsonData = getParamJsonData(vo.getData());
        List<TransportSgDwNzxBoat> transportSgDwNzxBoats= JSONUtil.toList(paramJsonData,TransportSgDwNzxBoat.class);
        if (transportSgDwNzxBoats!=null && transportSgDwNzxBoats.size()>0)
        {
            transportSgDwNzxBoats.forEach(i -> {
                i.setUpdateTime(new Date());
            });
            TransportSgDwNzxBoat sgDwNzxBoat=transportSgDwNzxBoats.get(0);
            //获取完工时间
            Date date=sgDwNzxBoat.getNzxAendtm();
            //1.先判断数据所属年和季度，如果不存在，则直接插入或更新
           LambdaQueryWrapper<TransportSgSyncStatus> lambdaQueryWrapper=new LambdaQueryWrapper<>();
           lambdaQueryWrapper.eq(TransportSgSyncStatus::getSyncYear,DateUtils.getYear(date));
            lambdaQueryWrapper.eq(TransportSgSyncStatus::getSyncQuarter,DateUtils.getSeason(date));
            lambdaQueryWrapper.eq(TransportSgSyncStatus::getIsDeleted,0);
            TransportSgSyncStatus sgSyncStatuses= sgSyncStatusService.getOne(lambdaQueryWrapper);
            if (sgSyncStatuses!=null) {
                //有状态表数据
                if (sgSyncStatuses.getSyncStatus()==1) { //含有结束标识，开始校验分组后数据是否已经不允许提交
                        LambdaQueryWrapper<TransportSgBaseNumber> numberLambdaQueryWrapper = new LambdaQueryWrapper<>();
                        numberLambdaQueryWrapper.eq(TransportSgBaseNumber::getIsDeleted, 0);
                        numberLambdaQueryWrapper.eq(TransportSgBaseNumber::getBaseNotup, 1);
                        List<TransportSgBaseNumber> islocks = baseNumberService.list(numberLambdaQueryWrapper);
                        if (islocks != null && islocks.size() > 0) {
                            //已锁数据分组
                            Function<TransportSgDwNzxBoat, List<Object>> compositeKey = boats ->
                                    Arrays.asList(boats.getNzxType(), boats.getNzxVslcnname(), boats.getNzxAgent());
                            Map<List<Object>, List<TransportSgDwNzxBoat>> updata = transportSgDwNzxBoats.stream().collect(Collectors.groupingBy(compositeKey, Collectors.toList()));

                            Function<TransportSgBaseNumber, List<Object>> lockDataKey = boats ->
                                    Arrays.asList(boats.getBaseType(), boats.getBaseBoatName(), boats.getBaseCompyName());
                            Map<List<Object>, List<TransportSgBaseNumber>> lockData = islocks.stream().collect(Collectors.groupingBy(lockDataKey, Collectors.toList()));

                            if (updata != null && updata.keySet() != null && updata.keySet().size() > 0) {
                                List<Long> stringsFilterAll =new ArrayList<>();
                                List<TransportSgDwNzxBoat> needSaves = new ArrayList<>();
                                updata.keySet().forEach(i -> {
                                    if (lockData.keySet().contains(i))    //如果过滤数据一致
                                    {
                                        List<Long> stringsFilter = updata.get(i).stream().map(TransportSgDwNzxBoat::getNzxSchPkid).collect(Collectors.toList());
                                        stringsFilterAll.addAll(stringsFilter);

                                    } else {
                                        needSaves.addAll(updata.get(i));
                                    }
                                });
                                needSaves.forEach(i->{
                                    //只要没锁定，还允许修改或新增，但给一个更新状态
                                    i.setTransUpdate(1);
                                });
                                boolean a = boatService.saveOrUpdateBatch(needSaves);
                                if (a) {
                                    if (stringsFilterAll!=null && stringsFilterAll.size()>0) {
                                        JSONArray jsonArray = JSONUtil.parseArray(stringsFilterAll);
                                        String encodedData = resultEncode(jsonArray.toString());
                                        return ReturnMsg.success("存储成功,且部分数据已锁，无法修改，无法修改的pkId列表为data", encodedData);
                                    }else {
                                        return ReturnMsg.success("存储成功");
                                    }
                                } else {
                                    if (stringsFilterAll!=null && stringsFilterAll.size()>0) {
                                        JSONArray jsonArray = JSONUtil.parseArray(stringsFilterAll);
                                        String encodedData = resultEncode(jsonArray.toString());
                                        return ReturnMsg.error("存储失败且部分数据已锁，无法修改，无法修改的pkId列表为data", encodedData);
                                    }
                                    else {
                                        return ReturnMsg.error("存储失败，请重新尝试");
                                    }
                                }
                            }
                        }
                }
            }
            else {
                //没有状态表数据
                TransportSgSyncStatus sgSyncStatusesNew= new TransportSgSyncStatus();
                sgSyncStatusesNew.setSyncId(snowflakeConfig.snowflakeId());
                sgSyncStatusesNew.setSyncYear(DateUtils.getYear(date));
                sgSyncStatusesNew.setSyncQuarter(DateUtils.getSeason(date));
                sgSyncStatusesNew.setSyncStatus(0);
                sgSyncStatusesNew.setSyncStartTime(new Date());
                sgSyncStatusesNew.setSyncUpdateStatus(0);
                sgSyncStatusService.saveOrUpdateByMultiId(sgSyncStatusesNew);
            }
        }

        boolean a= boatService.saveOrUpdateBatch(transportSgDwNzxBoats);
        if (a) {
            return ReturnMsg.success("存储成功");
        }else {
            return ReturnMsg.error("存储失败");
        }
    }


    @Autowired
    private TransportSgDwNzxContainersService containersService;
    /**
     *  船舶载运集装箱数据
     * @param vo
     * @return
     */
    @PostMapping("/putShipCarryingContainers")
    public ReturnMsg putShipCarryingContainers(@RequestBody @Validated ObjectBodyVo vo) {
        if (!checkDbKey(vo.getKey())) {
            return ReturnMsg.error(HttpStatus.BAD_REQUEST, "key值有误,请确认您输入的key值正确!");
        }
        String paramJsonData = getParamJsonData(vo.getData());
        List<TransportSgDwNzxContainers> transportSgDwNzxAgents= JSONUtil.toList(paramJsonData,TransportSgDwNzxContainers.class);
        if (transportSgDwNzxAgents!=null && transportSgDwNzxAgents.size()>0) {
            transportSgDwNzxAgents.forEach(i->{i.setUpdateTime(new Date());});
            boolean a = containersService.saveOrUpdateBatch(transportSgDwNzxAgents);
            if (a) {
                return ReturnMsg.success("存储成功");
            } else {
                return ReturnMsg.error("存储失败");
            }
        }else {
            return ReturnMsg.error("无传递数据");
        }

    }

    @Autowired
    private TransportSgDwNzxAgentService agentService;

    /**
     *  航运企业信息
     * @param vo
     * @return
     */
    @PostMapping("/putEnterpriseNames")
    public ReturnMsg putEnterpriseNames(@RequestBody @Validated ObjectBodyVo vo) {
        if (!checkDbKey(vo.getKey())) {
            return ReturnMsg.error(HttpStatus.BAD_REQUEST, "key值有误,请确认您输入的key值正确!");
        }
        String paramJsonData = getParamJsonData(vo.getData());
        List<TransportSgDwNzxAgent> transportSgDwNzxAgents= JSONUtil.toList(paramJsonData,TransportSgDwNzxAgent.class);
        if (transportSgDwNzxAgents!=null && transportSgDwNzxAgents.size()>0) {
            transportSgDwNzxAgents.forEach(i -> {
                i.setUpdateTime(new Date());
            });
            boolean a = agentService.saveOrUpdateBatch(transportSgDwNzxAgents);
            if (a) {
                return ReturnMsg.success("存储成功");
            } else {
                return ReturnMsg.error("存储失败");
            }
        }else {
            return ReturnMsg.error("无传递数据");
        }
    }


    @Autowired
    private TransportSgSyncStatusService sgSyncStatusService;




    @Autowired
    private TransportSgBaseNumberMapper baseNumberMapper;

    /**
     * 季度状态接口
     * @param vo
     * @return
     */
    @PostMapping("/puQuarterlyStatus")
    public ReturnMsg puQuarterlyStatus(@RequestBody @Validated ObjectBodyVo vo) {
        if (!checkDbKey(vo.getKey())) {
            return ReturnMsg.error(HttpStatus.BAD_REQUEST, "key值有误,请确认您输入的key值正确!");
        }
        String paramJsonData = getParamJsonData(vo.getData());
        TransportSgSyncStatus sgSyncStatuses= JSONUtil.toBean(paramJsonData,TransportSgSyncStatus.class);
        if(sgSyncStatuses!=null) {
            int pageSize = 1000;
            int count=1;
            if (sgSyncStatuses.getSyncStatus()==1 && sgSyncStatuses.getSyncUpdateStatus() == 0) {
                LambdaQueryWrapper<TransportSgBaseNumber> sgBaseNumberLambdaQueryWrapper=new LambdaQueryWrapper<>();
                sgBaseNumberLambdaQueryWrapper.eq(TransportSgBaseNumber::getBaseYear,sgSyncStatuses.getSyncYear());
                sgBaseNumberLambdaQueryWrapper.eq(TransportSgBaseNumber::getBaseQuarter,sgSyncStatuses.getSyncQuarter());
                baseNumberService.remove(sgBaseNumberLambdaQueryWrapper);
                Calendar startDate = DateUtils.getQuarterStart(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter());
                Calendar endDate = DateUtils.getQuarterEnd(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter());
                PageHelper.startPage(count, pageSize);
                Page<TransportSgBaseNumber> baseNumberPage = baseNumberMapper.getAllForCountData(startDate.getTime(), endDate.getTime());
                PageHelper.clearPage();
                if (baseNumberPage != null && baseNumberPage.size() > 0 && baseNumberPage.getResult() != null && baseNumberPage.getResult().size() > 0) {
                    if (baseNumberPage.getTotal() > pageSize) {
                        Long totalCount = baseNumberPage.getTotal();
                        Long totalPages = (totalCount + pageSize - 1) / pageSize;
                        for (int i = 1; i <= totalPages; i++) {
                            System.out.println("===  本次运行第："+i+"页");
                            PageHelper.startPage(i, pageSize);
                            Page<TransportSgBaseNumber> baseNumberPages = baseNumberMapper.getAllForCountData(startDate.getTime(), endDate.getTime());
                            if (i == 1) {
                                saveAllTransportSgBaseNumber(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter(), baseNumberPages.getResult(), null);
                            } else {
                                saveAllTransportSgBaseNumber(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter(), baseNumberPages.getResult(), null);
                            }
                            PageHelper.clearPage();
                        }
                    } else {
                        saveAllTransportSgBaseNumber(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter(), baseNumberPage.getResult(), null);
                    }
                }

                //所以企业数据合并成上港的
                LambdaQueryWrapper<TransportSgBaseNumber> lambdaQueryWrappera = new LambdaQueryWrapper();
                lambdaQueryWrappera.eq(TransportSgBaseNumber::getBaseYear, sgSyncStatuses.getSyncYear());
                lambdaQueryWrappera.eq(TransportSgBaseNumber::getBaseQuarter, sgSyncStatuses.getSyncQuarter());
                IPage<TransportSgBaseNumber> baseNumberPages = baseNumberMapper.selectPage(new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(count, pageSize), lambdaQueryWrappera);
                if (baseNumberPages.getTotal() > pageSize) {
                    Long totalCount = baseNumberPage.getTotal();
                    Long totalPages = (totalCount + pageSize - 1) / pageSize;
                    for (int i = 1; i <= totalPages; i++) {
                        if (i == 1) {
                            baseNumberPages.getRecords().forEach(j -> {
                                j.setBaseCompyName("上港集团");
                                j.setBaseHangId(j.getBaseId());
                                j.setBaseId(snowflakeConfig.snowflakeId());
                            });
                            baseNumberService.saveOrUpdateBatch(baseNumberPages.getRecords());
                        } else {
                            IPage<TransportSgBaseNumber> baseNumberPagesNow = baseNumberService.page(new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(i, pageSize), lambdaQueryWrappera);
                            baseNumberPagesNow.getRecords().forEach(j -> {
                                j.setBaseCompyName("上港集团");
                                j.setBaseHangId(j.getBaseId());
                                j.setBaseId(snowflakeConfig.snowflakeId());
                            });
                            baseNumberService.saveOrUpdateBatch(baseNumberPagesNow.getRecords());
                        }
                    }
                } else {
                    baseNumberPages.getRecords().forEach(i -> {
                        i.setBaseCompyName("上港集团");
                        i.setBaseHangId(i.getBaseId());
                        i.setBaseId(snowflakeConfig.snowflakeId());
                    });
                    baseNumberService.saveOrUpdateBatch(baseNumberPages.getRecords());
                }
            }
            else if (sgSyncStatuses.getSyncStatus()==1 && sgSyncStatuses.getSyncUpdateStatus() == 1){
                //只计算未锁定的，已锁定的不能删除
                LambdaQueryWrapper<TransportSgBaseNumber> sgBaseNumberLambdaQueryWrapper=new LambdaQueryWrapper<>();
                sgBaseNumberLambdaQueryWrapper.eq(TransportSgBaseNumber::getBaseYear,sgSyncStatuses.getSyncYear());
                sgBaseNumberLambdaQueryWrapper.eq(TransportSgBaseNumber::getBaseQuarter,sgSyncStatuses.getSyncQuarter());
                sgBaseNumberLambdaQueryWrapper.isNull(TransportSgBaseNumber::getBaseNotup);
                baseNumberService.remove(sgBaseNumberLambdaQueryWrapper);

                Calendar startDate = DateUtils.getQuarterStart(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter());
                Calendar endDate = DateUtils.getQuarterEnd(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter());
                PageHelper.startPage(count, pageSize);
                Page<TransportSgBaseNumber> baseNumberPage = baseNumberMapper.getAllForCountDataForUpdate(startDate.getTime(), endDate.getTime());
                PageHelper.clearPage();
                if (baseNumberPage != null && baseNumberPage.size() > 0 && baseNumberPage.getResult() != null && baseNumberPage.getResult().size() > 0) {
                    if (baseNumberPage.getTotal() > pageSize) {
                        Long totalCount = baseNumberPage.getTotal();
                        Long totalPages = (totalCount + pageSize - 1) / pageSize;
                        for (int i = 1; i <= totalPages; i++) {
                            System.out.println("===  本次运行第："+i+"页");
                            PageHelper.startPage(i, pageSize);
                            Page<TransportSgBaseNumber> baseNumberPages = baseNumberMapper.getAllForCountDataForUpdate(startDate.getTime(), endDate.getTime());
                            if (i == 1) {
                                saveAllTransportSgBaseNumber(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter(), baseNumberPages.getResult(), null);
                            } else {
                                saveAllTransportSgBaseNumber(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter(), baseNumberPages.getResult(), null);
                            }
                            PageHelper.clearPage();
                        }
                    } else {
                        saveAllTransportSgBaseNumber(sgSyncStatuses.getSyncYear(), sgSyncStatuses.getSyncQuarter(), baseNumberPage.getResult(), null);
                    }
                }
                //所以企业数据合并成上港的
                LambdaQueryWrapper<TransportSgBaseNumber> lambdaQueryWrappera = new LambdaQueryWrapper();
                lambdaQueryWrappera.eq(TransportSgBaseNumber::getBaseYear, sgSyncStatuses.getSyncYear());
                lambdaQueryWrappera.eq(TransportSgBaseNumber::getBaseQuarter, sgSyncStatuses.getSyncQuarter());
                sgBaseNumberLambdaQueryWrapper.ne(TransportSgBaseNumber::getBaseNotup,1);
                IPage<TransportSgBaseNumber> baseNumberPages = baseNumberMapper.selectPage(new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(count, pageSize), lambdaQueryWrappera);
                if (baseNumberPages.getTotal() > pageSize) {
                    Long totalCount = baseNumberPage.getTotal();
                    Long totalPages = (totalCount + pageSize - 1) / pageSize;
                    for (int i = 1; i <= totalPages; i++) {
                        if (i == 1) {
                            baseNumberPages.getRecords().forEach(j -> {
                                j.setBaseCompyName("上港集团");
                                j.setBaseHangId(j.getBaseId());
                                j.setBaseId(snowflakeConfig.snowflakeId());
                            });
                            baseNumberService.saveOrUpdateBatch(baseNumberPages.getRecords());
                        } else {
                            IPage<TransportSgBaseNumber> baseNumberPagesNow = baseNumberService.page(new com.baomidou.mybatisplus.extension.plugins.pagination.Page<>(i, pageSize), lambdaQueryWrappera);
                            baseNumberPagesNow.getRecords().forEach(j -> {
                                j.setBaseCompyName("上港集团");
                                j.setBaseHangId(j.getBaseId());
                                j.setBaseId(snowflakeConfig.snowflakeId());
                            });
                            baseNumberService.saveOrUpdateBatch(baseNumberPagesNow.getRecords());
                        }
                    }
                } else {
                    baseNumberPages.getRecords().forEach(i -> {
                        i.setBaseCompyName("上港集团");
                        i.setBaseHangId(i.getBaseId());
                        i.setBaseId(snowflakeConfig.snowflakeId());
                    });
                    baseNumberService.saveOrUpdateBatch(baseNumberPages.getRecords());
                }
            }

        }else {
            return ReturnMsg.error("未获取到传递的data数据信息");
        }
        boolean a = sgSyncStatusService.saveOrUpdateByMultiId(sgSyncStatuses);
        if (a) {
            return ReturnMsg.success("存储成功");
        } else {
            return ReturnMsg.error("存储失败");
        }
    }




    /**
     * 组装数据
     * @param sgBaseNumberList
     * @return
     */
    List<TransportSgBaseNumber>   saveAllTransportSgBaseNumber(Integer year,Integer q,List<TransportSgBaseNumber> sgBaseNumberList,String compyName)
    {
        System.out.println("===   本次查询数据："+sgBaseNumberList.toString());
        //先分组
        Map<Integer,List<TransportSgBaseNumber>> listMap=new HashMap<>();
        sgBaseNumberList.stream().forEach(i->{
            String hashKey=i.getBaseBoatName()+i.getBaseCompyName()+i.getBaseType();
            List<TransportSgBaseNumber> baseNumbers= listMap.get(hashKey.hashCode());
            if (baseNumbers!=null && baseNumbers.size()>0) {
                baseNumbers.add(i);
                listMap.put(hashKey.hashCode(),baseNumbers);
            }else {
                List<TransportSgBaseNumber> sgBaseNumberList1=new ArrayList<>();
                sgBaseNumberList1.add(i);
                listMap.put(hashKey.hashCode(),sgBaseNumberList1);
            }
        });
        List<TransportSgBaseNumber> needSaveData=new ArrayList<>();
        //再合并
        listMap.keySet().forEach(i->{
            List<TransportSgBaseNumber> baseNumbers= listMap.get(i);
            List<TransportSgBaseNumber> base20E= baseNumbers.stream().filter(j->"E".equals(j.getColumn2())).filter(j->"20".equals(j.getColumn1())).collect(Collectors.toList());
            List<TransportSgBaseNumber> base40E= baseNumbers.stream().filter(j->"E".equals(j.getColumn2())).filter(j->"40".equals(j.getColumn1())).collect(Collectors.toList());
            List<TransportSgBaseNumber> base45E= baseNumbers.stream().filter(j->"E".equals(j.getColumn2())).filter(j->"45".equals(j.getColumn1())).collect(Collectors.toList());
            List<TransportSgBaseNumber> base20F= baseNumbers.stream().filter(j->"F".equals(j.getColumn2())).filter(j->"20".equals(j.getColumn1())).collect(Collectors.toList());
            List<TransportSgBaseNumber> base40F= baseNumbers.stream().filter(j->"F".equals(j.getColumn2())).filter(j->"40".equals(j.getColumn1())).collect(Collectors.toList());
            List<TransportSgBaseNumber> base45F= baseNumbers.stream().filter(j->"F".equals(j.getColumn2())).filter(j->"45".equals(j.getColumn1())).collect(Collectors.toList());
            TransportSgBaseNumber sgBaseNumber= new TransportSgBaseNumber();
            BeanUtils.copyProperties(baseNumbers.get(0), sgBaseNumber);
            if (base20E!=null && base20E.size()>0) {
                sgBaseNumber.setBaseBoxes20e(new BigDecimal(base20E.get(0).getColumn3()));
            }
            if (base40E!=null && base40E.size()>0) {
                sgBaseNumber.setBaseBoxes40e(new BigDecimal(base40E.get(0).getColumn3()));
            }
            if (base45E!=null && base45E.size()>0) {
                sgBaseNumber.setBaseBoxes45e(new BigDecimal(base45E.get(0).getColumn3()));
            }
            if (base20F!=null && base20F.size()>0) {
                sgBaseNumber.setBaseBoxes20f(new BigDecimal(base20F.get(0).getColumn3()));
            }
            if (base40F!=null && base40F.size()>0) {
                sgBaseNumber.setBaseBoxes40f(new BigDecimal(base40F.get(0).getColumn3()));
            }
            if (base45F!=null && base45F.size()>0) {
                sgBaseNumber.setBaseBoxes45f(new BigDecimal(base45F.get(0).getColumn3()));
            }
            if (StringUtils.isNotEmpty(compyName))
            {
                sgBaseNumber.setBaseCompyName(compyName);
            }
            sgBaseNumber.setBaseYear(year);
            sgBaseNumber.setBaseQuarter(q);
            //设置hascode
            sgBaseNumber.setColumn1(String.valueOf(i));
            needSaveData.add(sgBaseNumber);
        });

        System.out.println("===   合并属性合并后数据："+needSaveData.toString());
        ////已查到 分组数据，再次合并成一条数据
        if (needSaveData!=null && needSaveData.size()>0) {

            Map<String,TransportSgBaseNumber> hashCodes = needSaveData.stream().collect(Collectors.toMap(TransportSgBaseNumber::getColumn1, Function.identity()));
            LambdaQueryWrapper<TransportSgBaseNumber> lambdaQueryWrapper= new LambdaQueryWrapper<>();
            lambdaQueryWrapper.in(TransportSgBaseNumber::getColumn1,hashCodes.keySet());
            lambdaQueryWrapper.ne(TransportSgBaseNumber::getBaseCompyName,"上港集团");
            //获取已存储后的数据
            List<TransportSgBaseNumber> sgBaseNumbers =    baseNumberService.list(lambdaQueryWrapper);
            if (sgBaseNumbers!=null && sgBaseNumbers.size()>0)
            {
                List<TransportSgBaseNumber> needSaveDatas=onlySum(needSaveData,sgBaseNumbers);
                if (needSaveDatas!=null && needSaveDatas.size()>0)
                {
                    System.out.println("===   存储数据："+needSaveDatas.toString());
                    needSaveDatas.forEach(i->{
                        i.setUpdateTime(new Date());
                    });
                    baseNumberService.saveOrUpdateBatch(needSaveDatas);
                    return needSaveDatas;
                }
            }
            else {  //未查到
                needSaveData.forEach(i->{
                    i.setBaseId(snowflakeConfig.snowflakeId());
                    i.setUpdateTime(new Date());
                });
                System.out.println("===   存储数据："+needSaveData.toString());
                baseNumberService.saveOrUpdateBatch(needSaveData);
            }
        }
        return needSaveData;

    }

    /**
     * 属性合并并条目合并
     * @param centers  旧的
     * @param list  新的
     * @return
     */
    private List<TransportSgBaseNumber> onlySum(List<TransportSgBaseNumber> centers, List<TransportSgBaseNumber> list) {
        if (centers==null || centers.size()<1)
        {
            try {
                return ObjOPUtil.deepCopy(list);

            } catch (IOException e) {
                e.printStackTrace();
            } catch (ClassNotFoundException e) {
                e.printStackTrace();
            }
        }
        List<TransportSgBaseNumber> list1=new ArrayList<>();
        List<String> common=centers.stream().map(TransportSgBaseNumber::getColumn1).collect(Collectors.toList());
        List<String> newDataCode=list.stream().map(TransportSgBaseNumber::getColumn1).collect(Collectors.toList());
        common.retainAll(newDataCode);
        Map<String,TransportSgBaseNumber> old=centers.stream().collect(Collectors.toMap(TransportSgBaseNumber::getColumn1,Function.identity()));
        Map<String,TransportSgBaseNumber> newData=list.stream().collect(Collectors.toMap(TransportSgBaseNumber::getColumn1,Function.identity()));
        //找到旧的不存在的
        List<TransportSgBaseNumber>  newDatas=list.stream().filter(i->!old.containsKey(i.getColumn1())).collect(Collectors.toList());
        list1.addAll(newDatas);

        for (String oldCode:old.keySet() ){
            if (common.contains(oldCode))
            {         //存储公共的
                TransportSgBaseNumber center=new ObjOPUtil<TransportSgBaseNumber>().fildMergeNewBean(newData.get(oldCode),old.get(oldCode));
                list1.add(center);
            }else {
                TransportSgBaseNumber sgBaseNumber =old.get(oldCode);
                sgBaseNumber.setBaseId(snowflakeConfig.snowflakeId());
                list1.add(sgBaseNumber);
            }
        };
        return list1;
    }
}
